QuestItemBar = LibStub("AceAddon-3.0"):NewAddon("QuestItemBar", "AceConsole-3.0", "AceEvent-3.0", "AceBucket-3.0")

local Gratuity = LibStub:GetLibrary("LibGratuity-3.0")

--LOCAL TABLES AND VARS
local itemUpdateNeeded = false
local questItemLinks = {}
local buttonSize = 36

--GUI FUNCTIONS
function QuestItemBar:CreateAnchor()
	local f = CreateFrame("Frame",self.name.."Anchor",UIParent)
	
	f.buttons = {}

	--f:SetToplevel(1)
	--f:SetFrameStrata("MEDIUM")
	f:SetClampedToScreen(true)
	f:SetHeight(buttonSize)
	f:SetWidth(buttonSize)

	f:SetAlpha(self.db.profile.bar.alpha)
	f:SetScale(self.db.profile.bar.scale)
	f:SetMovable(not self.db.profile.bar.locked)
	f:SetPoint(self.db.profile.bar.relativePoint, self.db.profile.bar.x, self.db.profile.bar.y)

	local overlay = CreateFrame("Frame", f:GetName() .. "Overlay", f)
	f.overlay = overlay
	overlay:EnableMouse(true)
	overlay:SetBackdrop({
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		tile = true,
		tileSize = 16,
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		edgeSize = 16,
		insets = {left = 3, right = 3, top = 3, bottom = 3}
	})
	overlay:SetBackdropColor(0,0,0,1)
	overlay:SetBackdropBorderColor(0.3,0.3,0.3,1)
	
	overlay:SetFrameLevel(f:GetFrameLevel() + 10)
	overlay:ClearAllPoints()
	overlay:SetAllPoints(f)
	if self.db.profile.bar.locked then overlay:Hide()
	else overlay:Show() end
	
	overlay:EnableMouse(true)
	overlay:SetScript("OnMouseDown",function(frame, button)
		local parent = frame:GetParent()
		if button == "LeftButton" and parent:IsMovable() then
			parent:StartMoving()
		end
	end)
	overlay:SetScript("OnMouseUp",function(frame, button)
		if button == "LeftButton" then
			local parent = frame:GetParent()
			parent:StopMovingOrSizing()
			local _, _, relativePoint, x, y = parent:GetPoint()
			QuestItemBar:SaveBarPosition(relativePoint, x, y)
		elseif button == "RightButton" then
			QuestItemBar:ChangeBarDirection()
			QuestItemBar:UpdateBar()
		end
	end)
	
	return f
end

local function buttonOnEnter(self)
	QuestItemBar.anchor:SetAlpha(QuestItemBar.db.profile.bar.alphaHover)
	if ( GetCVar("UberTooltips") == "1" ) then
		GameTooltip_SetDefaultAnchor(GameTooltip, self)
	else
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	end
	GameTooltip:SetHyperlink(self:GetAttribute("item"))
end
local function buttonOnLeave()
	QuestItemBar.anchor:SetAlpha(QuestItemBar.db.profile.bar.alpha)
	GameTooltip:Hide() 
end
		
function QuestItemBar:CreateItemButton(index, parent)
	if parent.buttons[index] then
		return parent.buttons[index]
	end

	local buttonName = self.name.."Button"..index
	local button = CreateFrame("Button", buttonName, parent, "SecureActionButtonTemplate")
	
	button:SetWidth(buttonSize)
	button:SetHeight(buttonSize)

	button:SetScript("OnEnter", buttonOnEnter)
	button:SetScript("OnLeave", buttonOnLeave)
	button:SetHighlightTexture([[Interface\Buttons\ButtonHilight-Square]])
	
	button.icon = button:CreateTexture(buttonName.."Icon","BORDER")
	button.icon:SetAllPoints()
	
	--button.border = button:CreateTexture(buttonName.."Border","OVERLAY")
	--button.border:SetTexture([[Interface\Buttons\UI-ActionButton-Border]])
		
	button.cooldown = CreateFrame("Cooldown", buttonName.."Cooldown", button, "CooldownFrameTemplate")
	button.cooldown:SetAllPoints()
	
	button.count = button:CreateFontString(buttonName.."Count", "OVERLAY", "NumberFontNormal")
	button.count:SetPoint("BOTTOMRIGHT", button.icon, -1, 1)
	
	button.flash = button:CreateTexture(buttonName.."Flash","OVERLAY")
	button.flash:Hide()
	
	if self.LBFGroup then
		self.LBFGroup:AddButton(button)
	end

	parent.buttons[index] = button
	return parent.buttons[index]
end

function QuestItemBar:SetButtonPoint(index)
	local dir = self.db.profile.bar.direction
	local padding = self.db.profile.bar.padding
	local button = self.anchor.buttons[index]
	local parent
	
	if index == 1 then
		parent = self.anchor
		if QuestItemBar.db.profile.bar.locked then
			self.anchor.overlay:Hide()
			padding = -buttonSize
		else
			self.anchor.overlay:Show()
		end
	elseif index > 1 then
		parent = self.anchor.buttons[index - 1]
	end
	
	button:ClearAllPoints() 
	if dir == "RIGHT" then
		button:SetPoint("LEFT", parent, "RIGHT", padding, 0)
	elseif dir == "LEFT" then
		button:SetPoint("RIGHT", parent, "LEFT", -padding, 0)
	elseif dir == "UP" then
		button:SetPoint("BOTTOM", parent, "TOP", 0, padding)
	elseif dir == "DOWN" then
		button:SetPoint("TOP", parent, "BOTTOM", 0, -padding)
	end
end

function QuestItemBar:AddButton(index, itemLink, bag, slot)
	local button = self:CreateItemButton(index, self.anchor)
	
	local texture, itemCount, locked, quality, readable = GetContainerItemInfo(bag, slot)
	
	button.icon:SetTexture(texture)
	button.count:SetText(itemCount and itemCount > 1 and itemCount or "")
	
	button:SetAttribute("type*","item")
	button:SetAttribute("item",itemLink)
	
	self:SetButtonPoint(index)
		
	button:Show()
end
--END GUI FUNCTIONS

--LOCAL FUNCTIONS
local function CheckIfQuestItem(itemLink)
	local name, link, quality, iLevel, reqLevel, itemType, itemSubType, maxStack, equipSlot, texture = GetItemInfo(itemLink)
	
	Gratuity:SetHyperlink(itemLink)
	
	if itemType == "Quest" or itemSubType == "Quest" or Gratuity:Find(ITEM_BIND_QUEST) then
		local usable = false
		if Gratuity:Find(ITEM_SPELL_TRIGGER_ONUSE) or Gratuity:Find(ITEM_STARTS_QUEST) then
			usable = true
		end
		return true, usable
	end
end

local function CheckBag(bagIndex)
	local itemLink
	for bagSlot = 1, GetContainerNumSlots(bagIndex) do
		itemLink = GetContainerItemLink(bagIndex, bagSlot)
		
		if itemLink then
			local isQuestItem, isUsable = CheckIfQuestItem(itemLink)
			if isQuestItem then
				tinsert(questItemLinks, { itemLink = itemLink, bag = bagIndex, slot = bagSlot, usable = isUsable })
			end
		end
	end
end
--END LOCAL FUNCTIONS

function QuestItemBar:OnInitialize()
    -- Called when the addon is loaded
	self.db = LibStub("AceDB-3.0"):New("QuestItemBarDB", self.defaults, "Default")

	self:SetupOptions()
	self:LoadLDB()
	
	self.anchor = self:CreateAnchor()
	
	self:LoadLBF()
end

function QuestItemBar:OnEnable()
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterBucketEvent("BAG_UPDATE", 0.2)
	self:RegisterBucketEvent("ACTIONBAR_UPDATE_COOLDOWN", 0.2)
end

function QuestItemBar:BAG_UPDATE()
	self:CheckInvetoryItems()
end

function QuestItemBar:PLAYER_REGEN_ENABLED()
	--self:ScheduleTimer("CheckInvetoryForItems", self.CheckInvetoryItems, 0.5, self)
	if itemUpdateNeeded then
		self:CheckInvetoryItems(true)
		itemUpdateNeeded = false
	end
end

function QuestItemBar:ACTIONBAR_UPDATE_COOLDOWN()
	local bag, slot;
	for index,button in ipairs(self.anchor.buttons) do
		if button:IsShown() then
			local itemLink = button:GetAttribute("item")
			CooldownFrame_SetTimer(button.cooldown, GetItemCooldown(itemLink))
		end
	end
end

function QuestItemBar:CheckInvetoryItems(checkInventory)
	if InCombatLockdown() then
		itemUpdateNeeded = true
		return
	end

	--self:Print("CheckInvetoryItems")
	
	questItemLinks = {}
	
	--Check bags
	for bagIndex = 0, 4 do
		CheckBag(bagIndex)
	end
	
	self:UpdateBar()
end

function QuestItemBar:UpdateBar()
	--self:Print("UpdateBar")
	
	local index = 1
	for _,t in ipairs(questItemLinks) do
		--QuestItemBar:Print("Item: " .. t.itemLink .. ", " .. t.bag .. ", " .. t.slot .. ", " .. tostring(t.usable))
		if not self.db.profile.bar.onlyUsable or t.usable then
			self:AddButton(index, t.itemLink, t.bag, t.slot)
			index = index + 1
		end
	end
	
	--Hide unused anchor.buttons
	for i = index, #self.anchor.buttons do
		self.anchor.buttons[i]:Hide()
	end
end

function QuestItemBar:SkinChanged(SkinID, Gloss, Backdrop, Group, Button, Colors)
	self.db.profile.skin.ID = SkinID
	self.db.profile.skin.Gloss = Gloss
	self.db.profile.skin.Backdrop = Backdrop
	self.db.profile.skin.Colors = Colors
end

function QuestItemBar:SaveBarPosition(relativePoint, x, y)
	self.db.profile.bar.relativePoint = relativePoint
	self.db.profile.bar.x = x
	self.db.profile.bar.y = y
end

function QuestItemBar:ChangeBarDirection(dir)
	if not dir then
		dir = self.db.profile.bar.direction
		
		if dir == "UP" then dir = "RIGHT"
		elseif dir == "RIGHT" then dir = "DOWN"
		elseif dir == "DOWN" then dir = "LEFT"
		elseif dir == "LEFT" then dir = "UP"
		end
	end
	
	self.db.profile.bar.direction = dir
end

--LibDataBroker
local LDB = LibStub("LibDataBroker-1.1", true)
--Ripped from Bartender4
function QuestItemBar:LoadLDB()
	local LDBObj = LibStub("LibDataBroker-1.1"):NewDataObject("QuestItemBar", {
		type = "launcher",
		label = "QuestItemBar",
		OnClick = function(_, msg)
			if msg == "LeftButton" then
				if QuestItemBar.anchor:IsShown() then
					QuestItemBar.anchor:Hide()
				else
					QuestItemBar.anchor:Show()
				end
			elseif msg == "RightButton" then
				if IsShiftKeyDown() then
					QuestItemBar:OpenConfig()
				else
					QuestItemBar:ToggleLock()
				end
			end
		end,
		icon = "Interface\\Icons\\inv_misc_bag_17",
		OnTooltipShow = function(tooltip)
			if not tooltip or not tooltip.AddLine then return end
			tooltip:AddLine("QuestItemBar")
			tooltip:AddLine("|cffffff00Click|r to toggle bar visibility")
			tooltip:AddLine("|cffffff00Right-click|r to toggle locked state and anchor")
			tooltip:AddLine("|cffffff00Shift-right-click|r to open the options menu")
		end,
	})
end

--LibButtonFacade
local LBF = LibStub("LibButtonFacade", true)
function QuestItemBar:LoadLBF()
	if LBF then
		local group = LBF:Group("QuestItemBar", "QuestItemBar1")
		
		group.SkinID = self.db.profile.skin.ID or "Zoomed"
		group.Backdrop = self.db.profile.skin.Backdrop
		group.Gloss = self.db.profile.skin.Gloss
		group.Colors = self.db.profile.skin.Colors
		
		LBF:RegisterSkinCallback("QuestItemBar", self.SkinChanged, self)
		
		--group:Skin(skin.ID, skin.Gloss, skin.Backdrop, skin.Colors or {})
		
		self.LBFGroup = group
	end
end
