--CONFIG
QuestItemBar.defaults = {
    profile = {
		skin = {
			ID = "DreamLayout",
			Gloss = 0,
			Backdrop = true,
			Colors = {},
		},
		bar = {
			relativePoint = "CENTER",
			x = 0,
			y = 0,
			direction = "RIGHT",
			alpha = 1,
			alphaHover = 1,
			scale = 1,
			padding = 1,
			locked = false,
			hidden = false,
			onlyUsable = false,
		},
		buttonSize = 36,
    },
}
--END CONFIG

--Ripped from Recount. Thanks!
local function deepCopy(object)
	local lookup_table = {}
	local function _copy(object)
		if type(object) ~= "table" then
			return object
		elseif lookup_table[object] then
			return lookup_table[object]
		end
		local new_table = {}
		lookup_table[object] = new_table
		for index, value in pairs(object) do
			new_table[_copy(index)] = _copy(value)
		end
		return setmetatable(new_table, getmetatable(object))
	end
	return _copy(object)
end

--OPTIONS
-- QuestItemBar.options = {
    -- name = "QuestItemBar Options",
    -- handler = QuestItemBar,
    -- type = 'group',
	-- args = {
		
	-- },
-- }

QuestItemBar.consoleOptions = {
	type = "group",
	name = "General Settings",
	handler = QuestItemBar,
	args = {
		lock = {
			order = 1,
			type = "toggle",
			name = "Lock bar",
			desc = "Lock bar and hide anchor.",
			set = "SetLock",
			get = function() return QuestItemBar.db.profile.bar.locked end,
		},
		onlyUsable = {
			order = 2,
			type = "toggle",
			name = "Show only usable items",
			desc = "Show only usable items and hide all other quest items.",
			set = "SetUsable",
			get = function() return QuestItemBar.db.profile.bar.onlyUsable end,
		},
		
		styleheader = {
			order = 10,
			type = "header",
			name = "Bar Style & Layout",
		},
		alpha = {
			order = 11,
			name = "Alpha",
			desc = "Configure the alpha of the bar.",
			type = "range",
			min = 0, max = 1, bigStep = 0.1,
			get = function() return QuestItemBar.db.profile.bar.alpha end,
			set = "SetAlpha",
		},
		alphaHover = {
			order = 12,
			name = "Alpha on hover",
			desc = "Configure the alpha of the bar when mouse is over.",
			type = "range",
			min = .1, max = 1, bigStep = 0.1,
			get = function() return QuestItemBar.db.profile.bar.alphaHover end,
			set = "SetAlphaHover",
		},
		scale = {
			order = 13,
			name = "Scale",
			desc = "Configure the scale of the bar.",
			type = "range",
			min = .1, max = 2, step = 0.05,
			get = function() return QuestItemBar.db.profile.bar.scale end,
			set = "SetScale",
		},
		padding = {
			order = 14,
			type = "range",
			name = "Padding",
			desc = "Configure the padding of the buttons.",
			min = -10, max = 20, step = 1,
			get = function() return QuestItemBar.db.profile.bar.padding end,
			set = "SetPadding",
		},
		direction = {
			order = 15,
			type = "select",
			name = "Direction",
			desc = "Configure the direction of the buttons from the anchor.",
			values = { UP = "Up", DOWN = "Down", LEFT = "Left", RIGHT = "Right" },
			get = function() return QuestItemBar.db.profile.bar.direction end,
			set = "SetDirection",
		},
		config = {
			name = "Config GUI",
			desc = "Open the blizzard configuration UI.",
			type = "execute",
			guiHidden = true,
			func = "OpenConfig",
		},
	},
}
QuestItemBar.options = deepCopy(QuestItemBar.consoleOptions)

-- QuestItemBar.options.args.config = {
	-- name = "Config GUI",
	-- desc = "Open the blizzard configuration UI",
	-- type = "func",
	-- func = "OpenConfig",
-- }
--END OPTIONS

function QuestItemBar:SetAlpha(_, value)
	self.db.profile.bar.alpha = value
	self.anchor:SetAlpha(value)
end

function QuestItemBar:SetAlphaHover(_, value)
	self.db.profile.bar.alphaHover = value
end

function QuestItemBar:SetScale(_, value)
	self.db.profile.bar.scale = value
	self.anchor:SetScale(value)
end

function QuestItemBar:SetPadding(_, value)
	self.db.profile.bar.padding = value
	self:UpdateBar()
end

function QuestItemBar:SetDirection(_, value)
	self.db.profile.bar.direction = value
	self:UpdateBar()
end

function QuestItemBar:SetLock(_, value)
	QuestItemBar.db.profile.bar.locked = value
	self.anchor:SetMovable(not value)
	self:UpdateBar()
end

function QuestItemBar:ToggleLock()
	self:SetLock(_, not self.db.profile.bar.locked)
end

function QuestItemBar:SetUsable(_, value)
	self.db.profile.bar.onlyUsable = value
	self:UpdateBar()
end

function QuestItemBar:ChatCommand(input)
	if not input or input:trim() == "" then
        self:OpenConfig()
    else
        self:Print(input)
		LibStub("AceConfigCmd-3.0").HandleCommand(QuestItemBar, "qib", "QuestItemBar", input)
    end
end

function QuestItemBar:OpenConfig()
	if InCombatLockdown() then return end
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame.Profiles)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
end

function QuestItemBar:SetupOptions()
	self.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self.options.args.profiles.order = 2
		
	LibStub("AceConfig-3.0"):RegisterOptionsTable(self.name, QuestItemBar.consoleOptions, {"QuestItemBar", "qib"})
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("QuestItemBar Profile", self.options.args.profiles)
	
	--self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(self.name, nil, nil, "general")
	--self.optionsFrame.Profiles = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("QuestItemBar Profile", "Profiles", self.name, "profiles")
	
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(self.name, "QuestItemBar")
	self.optionsFrame.Profiles = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("QuestItemBar Profile", "Profile", "QuestItemBar")
    	
	--self:RegisterChatCommand("QuestItemBar", "ChatCommand")
    --self:RegisterChatCommand("qib", "ChatCommand")
end
